/*
** Curve_3D.h
** 
** Made by damien
** Login   <damien@ortie.inrialpes.fr>
** 
** Started on  Wed Mar 26 16:22:43 2008 damien
** Last update Wed Mar 26 16:22:43 2008 damien
*/

#ifndef   	CURVE_3D_H_
# define   	CURVE_3D_H_

#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
#include <vector>
using namespace std;

#include <V_3D.h>


class Curve_3D
{
 public:
  Curve_3D();
  Curve_3D(const Curve_3D& _curve);
  ~Curve_3D();

  int add_point(const V_3D& point);
  int add(const V_3D& point);
  int add(const Curve_3D& _curve);

  /// add the point at the begining
  int add_first(const V_3D& point);

  int size() const;
  int destroy();

  V_3D& get(int k_index);
  V_3D  get(int k_index) const;
  V_3D& operator[](int k_index);
  V_3D  operator[](int k_index) const;

  /// set the curve by a vector
  int set(const std::vector <V_3D>& _curve);
  /// set the curve by a vector
  int set(const std::vector <double>& _curve);

  double length() const;
  
  Curve_3D& operator=(const Curve_3D&);
  Curve_3D& operator=(const std::vector <V_3D>&);

  int resample_closed(int N);
  int resample(int N);

  std::vector <V_3D> get_diff_closed_curve() const;
  std::vector <V_3D> get_diff2_closed_curve() const;
  std::vector <double> get_curvature_closed_curve() const;

  std::vector <V_3D>& to_vector();
  const std::vector <V_3D>& to_vector() const;

  /// get a part of the Curve
  Curve_3D part(int k_0,int k_1) const;
  /// get the smallest part of a closed curve
  Curve_3D part_closed(int k_0,int k_1) const;

  /// resample a portion of the curve which is supposed to be closed
  Curve_3D resample_part_closed(int k_0,int k_1,int N);

  /// load a curve_3D file
  int load_curve(const char* filename);
  /// save a curve_3D file
  int save_curve(const char* filename) const;


  /// test equality
  friend bool operator==(const Curve_3D&, const Curve_3D& );
  /// test equality
  friend bool operator!=(const Curve_3D&, const Curve_3D& );

 private:

  std::vector <V_3D> data;

};


#endif 	    /* !CURVE_3D_H_ */
