
/*
**    Mesh Converter
**    Copyright (C) 2008 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



/*
** Edge.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Sat May 17 21:03:59 2008 damien
** Last update Sat May 17 21:03:59 2008 damien
*/

#ifndef   	EDGE_H_
# define   	EDGE_H_


#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>

#include <tinyXml/tinyxml.h>
#include <tinyXml/tinystr.h>

#include <vector>

/// Edge for the Mesh class
/**
   Pointer index for Edge class.
   
   Index of vertices [i0,i1]
   Index of Opposite faces [F0,F1]
   Index of following, previous edges [E0,E1]
   
 */

class Edge
{

public:

  //************************************************//
  //************************************************//
  //CONSTRUCTORS
  //************************************************//
  //************************************************//


  /// Classical constructor
  Edge();
  /// Copy constructor
  Edge(const Edge&);
  /// Destructor
  ~Edge();
  /// Set all index to -1
  int destroy();


  //************************************************//
  //************************************************//
  //Set
  //************************************************//
  //************************************************//

  /// set first vertex index
  int set_index_0(int i0);
  /// set second vertex index
  int set_index_1(int i1);
  /// set both vertices index
  int set_index(int i0,int i1);
  
  /// set first opposite face
  int set_face_0(int f0);
  /// set second opposite face
  int set_face_1(int f1);
  /// set both opposite face
  int set_face(int f0,int f1);

  /// set next edge
  int set_next_edge(int e0);
  /// set previous edge
  int set_previous_edge(int e1);
  /// set both edges
  int set_edges(int e0,int e1);


  /// set the corresponding half edge
  int set_half_edge(int e);


  //************************************************//
  //************************************************//
  //Access
  //************************************************//
  //************************************************//

  /// get first vertex index
  int  get_index_0() const;
  /// get first vertex index
  int& get_index_0();
  /// get second vertex index
  int  get_index_1() const;
  /// get second vertex index
  int& get_index_1();

  /// get first face index
  int  get_face_0() const;
  /// get first face index
  int& get_face_0();
  /// get second face index
  int  get_face_1() const;
  /// get second face index
  int& get_face_1();  

  /// get first edge index (next edge)
  int  get_edge_0() const;
  /// get first edge index (next edge)
  int& get_edge_0();
  /// get second edge index (previous edge)
  int  get_edge_1() const;
  /// get second edge index (previous edge)
  int& get_edge_1();  
  /// get next edge index
  int  get_next_edge_index() const;
  /// get next edge index
  int& get_next_edge_index();
  /// get previous edge index
  int  get_previous_edge_index() const;
  /// get previous edge index
  int& get_previous_edge_index();  


  /// get the corresponding half edge
  int get_half_edge() const;
  /// get the corresponding half edge
  int& get_half_edge();


  //************************************************//
  //************************************************//
  //OPERATORS
  //************************************************//
  //************************************************//

  //affectation operator
  const Edge& operator=(const Edge&);

  /// test equality
  friend bool operator==(const Edge&,const Edge&);
  /// test equality
  friend bool operator!=(const Edge&,const Edge&);
  /// Stream output
  friend std::ostream& operator << (std::ostream& flux,const Edge& );


  
private:

  /// index of vertex 0
  int index_0;
  /// index of vertex 1
  int index_1;
  /// opposite face in direction 01
  int face_01;
  /// opposite face in direction 10
  int face_10;
  /// index of next Edge
  int next_edge;
  /// index of previous Edge
  int previous_edge;
  /// index to opposite half edge
  int half_edge;

};

#endif 	    /* !EDGE_H_ */
