/*
** Joint.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Sun Jan 27 10:12:26 2008 damien
** Last update Sun Jan 27 10:12:26 2008 damien
*/

#ifndef   	JOINT_H_
# define   	JOINT_H_


#define MAX_NAME 100



#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>

#include <vector>
using namespace std;

#include <Matrix.h>
#include <V_3D.h>
#include <list>


using namespace std;

class Joint
{
 public:
  Joint();
  Joint(const Joint&);
  ~Joint();

  //name
  int set_name(const char* _name);
  int set_name(const std::string &name);
  char *get_name();

  //position
  int set_position(double x,double y,double z);
  int add_position(double x,double y,double z);
  int add_position(double *translation);
  int get_position(double *_position);
  double *get_position();

  //orientation
  int set_orientation(double t1,double t2,double t3);
  int set_orientation(double *t);
  int get_orientation(double *_orientation);
  double *get_orientation();

  int multiply_matrix(Matrix R);

  int set_matrix(Matrix M);
  Matrix get_matrix();
  int get_matrix(Matrix *_M);

  int set_father(Joint *_father);
  Joint* get_father();

  int add_son();
  Joint* get_son(int k);

  int get_N_son();
  int add_rotation(double *v,double theta);//rotation around an axis
					   //and an angle
  int add_rotation(double v0,double v1,double v2,double theta);

  int scale(double *s);
  int scale(double s0,double s1,double s2);
  int set_bind_pose(Matrix D);
  int set_position_bind(double x,double y,double z);
  int set_position_bind(double *t);
  Matrix get_bind_pose();
  int get_bind_pose(Matrix *_M);
  
  int set_access_number(int _access_number);
  int get_access_number();

  int set_son(int k,Joint *new_joint);

  int set_world_matrix(Matrix _world_matrix);
  Matrix get_world_matrix();
  int set_T(Matrix _T);
  Matrix get_T();


  friend ostream& operator << (ostream& flux, Joint *joint)
    {flux<<joint->get_name();return flux;}
  Joint& operator=(const Joint& J);


  double get_min_distance_to_bone(V_3D vertex);//get distance min
					       //between vertex and
					       //bone (all childrens)

  int get_level();
  int set_level(int _level);

  int set_weight_name(const char* weight_name);
  const char* get_weight_name() const;

 private:

  int destroy();

  Joint *father;
  int N_son;
  std::list <Joint*> son;

  char name[MAX_NAME];
  std::string weight_name;
  Matrix M;//position and orientation
  double orientation[3];
  Matrix bind_pose;//bind pose
  Matrix world_matrix;//direct access
  Matrix T;//world_matrix*bind_pose

  int access_number;
  int level;

};



#endif 	    /* !JOINT_H_ */
