/*
** V_3D.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Fri Jan 25 11:12:12 2008 damien
** Last update Fri Jan 25 11:12:12 2008 damien
*/

#ifndef   	V_3D_H_
# define   	V_3D_H_

#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
using namespace std;

class V_3D
{
 public:
  V_3D();
  V_3D(const V_3D &v_3D);
  V_3D(double x,double y,double z);
  ~V_3D();


  int set(double *v);
  int set(int k_dim,double value);
  int set(V_3D _V);
  int set(double x,double y,double z);

  double get(int k_dim) const;
  int    get(double *v) const;
  V_3D   get() const;
  double *get_pointer();



  double dot(V_3D _v) const;
  V_3D vector_prod(V_3D _v) const;
  double norm() const;
  /// return the normalized vector (if norm>epsilon)
  /** return the normalized vector if norm is >0, return vector
      (0,0,1) if the norm is null */
  V_3D normalized() const;
  double area(V_3D _v) const;
  double cos_angle(const V_3D& v) const;
  /// get acos( a/||a|| . b/||b|| )
  double angle(const V_3D& v)const;

  V_3D get_min_scalar(const V_3D& _v) const;
  V_3D get_max_scalar(const V_3D& _v) const;
  double get_max_of_coeff() const;
  double get_min_of_coeff() const;

  int equal(const V_3D& v,double epsilon) const;


  double tetrahedra_volume(const V_3D& p2,const V_3D& p3,const V_3D& p4) const;
  int get_barycenter_coordinates_tetrahedra(const V_3D& p1,const V_3D& p2,const V_3D& p3,const V_3D& p4,double *alpha,double *beta,double *gamma,double *delta) const;
  int is_inside_volume(const V_3D& p1,const V_3D& p2,const V_3D& p3,const V_3D& p4) const;
  double distance_to_oriented_plane(const V_3D& n,const V_3D& x0) const;

  /// get the angle between the vector and the plane defined by the
  /// normal normal_to_plane
  /** cos(Angle) = <u,(u-<u,n>)/||u-<u,n>||>  */
  double cos_angle_to_plane_with_normal(const V_3D& normal_to_plane) const;


  V_3D& operator=(const V_3D&);
  V_3D& operator+=(const V_3D&);
  V_3D& operator/=(const double&);

  bool operator>=(const V_3D& ) const;
  bool operator>(const V_3D &) const;
  bool operator<=(const V_3D &) const;
  bool operator<(const V_3D &) const;

  friend V_3D operator+(const V_3D& v1,const V_3D& v2);
  friend V_3D operator-(const V_3D& v1,const V_3D& v2);
  friend V_3D operator*(const V_3D& v1,const double& alpha);
  friend V_3D operator*(const double& alpha,const V_3D& v1);
  friend V_3D operator/(const V_3D& v1,const double& alpha);
  friend bool operator==(const V_3D& v1,const V_3D& v2);
  friend bool operator!=(const V_3D& v1,const V_3D& v2);

  double& operator[](int index);
  double operator[](int index) const;

  friend ostream& operator << (ostream& flux, V_3D _v);

 private:

  double V[3];
};


#endif 	    /* !V_3D_H_ */
