/*
**    Mesh Converter
**    Copyright (C) 2008 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/*
** Vector_parameter.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Fri Jun  6 19:54:01 2008 damien
** Last update Fri Jun  6 19:54:01 2008 damien
*/

#ifndef   	VECTOR_PARAMETER_H_
# define   	VECTOR_PARAMETER_H_

#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
#include <vector>

#include <int_vector.h>
#include <double_vector.h>

/// General vector of int and double
/**
   General Multiple vector use.
   Container of a set of vector of integer
                  set of vector of double
   Number and dimension of each is let free
*/

class Vector_parameter
{
public:

  //************************************************//
  //************************************************//
  //CONSTRUCTORS
  //************************************************//
  //************************************************//

  /// classical constructor
  Vector_parameter();
  /// copy constructor
  Vector_parameter(const Vector_parameter& );
  /// classical destructor
  ~Vector_parameter();
  
  /// empty every vector
  int destroy();


  //************************************************//
  //************************************************//
  //Size
  //************************************************//
  //************************************************//

  /// get number of int_vector
  int size_int() const;
  /// get number of double_vector
  int size_double() const;
  /// resize the number of int_vector
  int resize_int(int N_size);
  /// resize the number of double_vector
  int resize_double(int N_size);

  /// get size of the int_vector at index k_vector
  int size_int(int k_vector) const;
  /// get size of the double vector
  int size_double(int k_vector) const;
  /// resize the int_vector[k_vector]
  int resize_int(int k_vector,int N_size);
  /// resize the double_vector[k_vector]
  int resize_double(int k_vector,int N_size);

  //************************************************//
  //************************************************//
  //Access
  //************************************************//
  //************************************************//

  /// get the int_vector at index k_index
  const int_vector& get_int(int k_index) const;
  int_vector      & get_int(int k_index);

  /// get the double_vector at index k_index
  const double_vector& get_double(int k_index) const;
  double_vector      & get_double(int k_index);

  /// get the int value from vector k_vector at index k_index
  int& get_int(int k_vector,int k_index);
  int  get_int(int k_vector,int k_index) const;

  /// get the double value from vector k_vector at index k_index
  double& get_double(int k_vector,int k_index);
  double  get_double(int k_vector,int k_index) const;


  //************************************************//
  //************************************************//
  //Operator
  //************************************************//
  //************************************************//

  /// Equal
  Vector_parameter& operator=(const Vector_parameter& );

private:

  /// set of vectors
  std::vector <int_vector> vector_int;
  std::vector <double_vector> vector_double;

};



#endif 	    /* !VECTOR_PARAMETER_H_ */
