/*
**    Mesh Converter
**    Copyright (C) 2008  Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <Edge.h>



Edge::Edge():
  index_0(-1),index_1(-1),face_01(-1),face_10(-1),next_edge(-1),previous_edge(-1),half_edge(-1)
{}

Edge::Edge(const Edge& e)
{
  index_0       = e.index_0;
  index_1       = e.index_1;
  face_01       = e.face_01;
  face_10       = e.face_10;
  next_edge     = e.next_edge;
  previous_edge = e.previous_edge;
  half_edge     = e.half_edge;
}
Edge::~Edge(){destroy();}
int Edge::destroy()
{
  index_0=-1;
  index_1=-1;
  face_01=-1;
  face_10=-1;
  next_edge=-1;
  previous_edge=-1;
  half_edge=-1;

  return 0;
}




int Edge::set_index_0(int i0){index_0=i0;return 0;}
int Edge::set_index_1(int i1){index_1=i1;return 0;}
int Edge::set_index(int i0,int i1){index_0=i0;index_1=i1;return 0;}

int Edge::set_face_0(int f0){face_01=f0;return 0;}
int Edge::set_face_1(int f1){face_10=f1;return 0;}
int Edge::set_face(int f0,int f1){face_01=f0;face_10=f1;return 0;}

int Edge::set_next_edge(int e0){next_edge=e0;return 0;}
int Edge::set_previous_edge(int e1){previous_edge=e1;return 0;}
int Edge::set_edges(int e0,int e1){next_edge=e0;previous_edge=e1;return 0;}

int Edge::set_half_edge(int e){half_edge=e;return 0;}




int  Edge::get_index_0() const {return index_0;}
int& Edge::get_index_0() {return index_0;}
int  Edge::get_index_1() const {return index_1;}
int& Edge::get_index_1() {return index_1;}

int  Edge::get_face_0() const {return face_01;}
int& Edge::get_face_0() {return face_01;}
int  Edge::get_face_1() const {return face_10;}
int& Edge::get_face_1() {return face_10;} 

int  Edge::get_edge_0() const {return next_edge;}
int& Edge::get_edge_0() {return next_edge;}
int  Edge::get_edge_1() const {return previous_edge;}
int& Edge::get_edge_1() {return previous_edge;}
int  Edge::get_next_edge_index() const{return next_edge;}
int& Edge::get_next_edge_index() {return next_edge;}
int  Edge::get_previous_edge_index() const {return previous_edge;}
int& Edge::get_previous_edge_index() {return previous_edge;}


int  Edge::get_half_edge() const{return half_edge;}
int& Edge::get_half_edge(){return half_edge;}


const Edge& Edge::operator=(const Edge& e)
{
  index_0       = e.index_0;
  index_1       = e.index_1;
  face_01       = e.face_01;
  face_10       = e.face_10;
  next_edge     = e.next_edge;
  previous_edge = e.previous_edge;
  half_edge     = e.half_edge;

  return *this;
}

bool operator==(const Edge& e1,const Edge& e2)
{
  if(e1.index_0!=e2.index_0)
    return 0;
  if(e1.index_1!=e2.index_1)
    return 0;
  if(e1.face_01!=e2.face_01)
    return 0;
  if(e1.face_10!=e2.face_10)
    return 0;
  if(e1.next_edge!=e2.next_edge)
    return 0;
  if(e1.previous_edge!=e2.previous_edge)
    return 0;
  if(e1.half_edge!=e2.half_edge)
    return 0;

  return 1;
}
bool operator!=(const Edge& e1,const Edge& e2)
{return !(e1==e2);}
std::ostream& operator << (std::ostream& flux,const Edge& e)
{
  flux<<"i ["<<e.index_0<<","<<e.index_1<<"] , f ["<<e.face_01<<","<<e.face_10<<"] , e ["<<e.next_edge<<","<<e.previous_edge<<"]"<<" h "<<e.half_edge;
  return flux;
}


