
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
#include <GL/glut.h>
#include <GL/glext.h>

#include <Mesh_object.h>
#include <File_parser.h>
#include <OpenGL_drawer.h>
#include <Navigator_tool.h>
#include <Skeleton.h>
#include <Skinned.h>



static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void motion_callback (int x, int y);
static void mouse_callback (int button, int action, int x, int y);


int setup_opengl(int argc,char **argv);
int deform_mesh();
int export_to_pov();
int draw_scene();
int draw_lines_screen();
int draw_selection_mode();

int load_model(const char* filename);





OpenGL_drawer openGL_drawer;

Navigator_tool navigator;
Skinned skinned;
Mesh_object *mesh=skinned.get_mesh();
Mesh_object *deformed_mesh=skinned.get_deformed_mesh();
Skeleton *skeleton = skinned.get_skeleton();
Skeleton *deformed_skeleton=skinned.get_deformed_skeleton();

Mesh_object chair;


int k_draw=0;

int main(int argc, char *argv[])
{
  cout<<"**hello**\n"<<endl;
  printf("run %s with %d parameters ... \n\n",argv[0],argc-1);


  //start OpenGL
  navigator.set_screen_size(800,800);
  setup_opengl(argc,argv);


  printf("\n\n==========================\n");
  printf("Space Displacement\n");
  printf("  W    R\n");
  printf("A S D  F\n\n");
  printf("and mouse with drag & drop\n\n");
  printf("\nView mode\n");
  printf("-----------\n");
  printf("W: wireframe mode\n");
  printf("F: filled mode\n");
  printf("\nSkeleton deformation\n");
  printf("-----------\n");
  printf("K/k: Joint View/Selection Mode\n");
  printf("SPACE: Select Joint/create new joint\n");
  printf("T: Translate Joint\n");
  printf("Y: Translate only this Joint\n");
  printf("A: Add son to selected joint\n");
  printf("X: Delete selected Joint\n");
  printf("H: Create new skeleton\n");
  printf("==========================\n\n");	 


  if(argc!=2)
    {printf("Error should be called with 1 argument: build_skeleton mesh_name \n");exit(-1);}
  load_model(argv[1]);
  deform_mesh();





  //run
  printf("**RUN**\n\n");
  glutMainLoop ();


  return 0;
}


int setup_opengl(int argc,char **argv)
{
  openGL_drawer.init_opengl(argc,argv,navigator);

  //callback
  glutDisplayFunc (display_callback);
  glutReshapeFunc (reshape_callback);
  glutKeyboardFunc(keyboard_callback);
  glutMotionFunc (motion_callback);
  glutMouseFunc (mouse_callback);

  return 0;
}


static void display_callback()
{
  double Z_MIN = 0.005;
  double Z_MAX = 300.0;

  //cout<<navigator.get_time()<<endl;

  glViewport (0, 0, navigator.get_X_screen(),navigator.get_Y_screen());
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);





  //view 1
  glViewport (0, (int)(navigator.get_Y_screen()*(1-navigator.get_percent_y())), (int)(navigator.get_X_screen()*navigator.get_percent_x()), (int)(navigator.get_Y_screen()*navigator.get_percent_y()));



  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (45, navigator.get_screen_ratio(), Z_MIN,Z_MAX);

  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glTranslatef(0,0,navigator.get_dist_obs());
  glRotatef(navigator.get_theta_1(),1.0,0.0,0.0);
  glRotatef(navigator.get_theta_2(),0.0,1.0,0.0);
  glTranslatef(navigator.get_tr_x(),navigator.get_tr_y(),navigator.get_tr_z());


  draw_scene();
  draw_selection_mode();
  //openGL_drawer.draw_local_basis(navigator);








  //view 2
  //********************//
  glViewport (0, 0, (int)(navigator.get_X_screen()*navigator.get_percent_x()), (int)(navigator.get_Y_screen()*(1-navigator.get_percent_y())));

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  
  glOrtho(navigator.get_dist_obs2() * navigator.get_percent_x()/(1.0-navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  -navigator.get_dist_obs2() * navigator.get_percent_x()/(1.0-navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  navigator.get_dist_obs2(),-navigator.get_dist_obs2(),Z_MIN,Z_MAX);



  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  //global
  glTranslatef(navigator.get_tr2_x(),navigator.get_tr2_y(),0);
  glTranslatef(0,0,navigator.get_dist_obs2());

  draw_scene();
  draw_selection_mode();


  //view 3
  //********************//
  glViewport ((int)(navigator.get_X_screen()*navigator.get_percent_x()),
	      (int)(navigator.get_Y_screen()*(1.0-navigator.get_percent_y())),
	      (int)(navigator.get_X_screen()*(1.0-navigator.get_percent_x())),
	      (int)(navigator.get_Y_screen()*navigator.get_percent_y()));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  glOrtho(navigator.get_dist_obs3() * (1.0-navigator.get_percent_x())/(navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  -navigator.get_dist_obs3() * (1.0-navigator.get_percent_x())/(navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  navigator.get_dist_obs3(),-navigator.get_dist_obs3(),Z_MIN,Z_MAX);
  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //global
  glTranslatef(navigator.get_tr3_x(),navigator.get_tr3_y(),0);
  glTranslatef(0,0,navigator.get_dist_obs3());
  glRotatef(-90.0,0.0,1.0,0.0);
  draw_scene();
  draw_selection_mode();




  //view 4
  //********************//
  glViewport ((int)(navigator.get_X_screen()*navigator.get_percent_x()),
	      0,
	      (int)(navigator.get_X_screen()*(1.0-navigator.get_percent_x())),
	      (int)(navigator.get_Y_screen()*(1.0-navigator.get_percent_y())));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  glOrtho(navigator.get_dist_obs4() * (1.0-navigator.get_percent_x())/(1.0-navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  -navigator.get_dist_obs4() * (1.0-navigator.get_percent_x())/(1.0-navigator.get_percent_y()) * navigator.get_screen_ratio(),
	  navigator.get_dist_obs4(),-navigator.get_dist_obs4(),Z_MIN,Z_MAX);
  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //global
  glTranslatef(navigator.get_tr4_x(),navigator.get_tr4_y(),0);
  glTranslatef(0,0,navigator.get_dist_obs4());
  glRotatef(90.0,1.0,0.0,0.0);
  draw_scene();
  draw_selection_mode();



  draw_lines_screen();


  //code d'erreur OpenGL
  GLenum errCode;
  const GLubyte *errString;
  if((errCode=glGetError()) != GL_NO_ERROR)
    {
      errString = gluErrorString(errCode);
      printf("ERROR OPENGL : %s\n",errString);
      exit(-2);
    }

  //redraw
  glFlush ();
  glutSwapBuffers ();

}

static void reshape_callback (int width, int height)
{
  navigator.set_screen_size(width,height);
}
static void keyboard_callback(unsigned char key,int x,int y)
{

  x=0;y=0;
  double d_L=0.001*(1+100*abs(navigator.get_dist_obs()));

  #define GLUT_SPACE 32
  File_parser parser;

  switch(key)
    {
    case 'Q'://quit
      printf("\n[EXIT OK: skeleton saved in [data/temp_skeleton.dae]]\n\n");
      exit(0);
      break;

    case 'w':
      navigator.go_forward(d_L);
      break;
    case 's':
      navigator.go_backward(d_L);
      break;
    case 'a':
      navigator.go_left(d_L);
      break;
    case 'd':
      navigator.go_right(d_L);
      break;
    case 'r':
      navigator.go_up(d_L);
      break;
    case 'f':
      navigator.go_down(d_L);
      break;
    case '+':
      mesh->increase_alpha(-0.01);
      *deformed_mesh=*mesh;
      break;
    case '-':
      mesh->increase_alpha( 0.01);
      *deformed_mesh=*mesh;
      break;
    case 'k'://selection mode
      navigator.get_selection_mode()==1?navigator.set_selection_mode(0):navigator.set_selection_mode(1);
      break;      
    case 'K'://view selection mode
      navigator.get_view_selection_mode()==1?navigator.set_view_selection_mode(0):navigator.set_view_selection_mode(1);
      break;
    case 'H'://change root
      if(navigator.is_change_root()==0)
	{
	  navigator.enable_change_root();
	  navigator.set_selection_mode(1);
	  printf("\n[WARNING SET SKELETON ROOT]\n\n");
	}
      else
	{
	  navigator.disable_change_root();
	  printf("</set skeleton root>\n");
	}
      break;
    case 'A'://add joint
      if(navigator.get_selected_joint()!=-1 && navigator.is_add_joint()==0 && navigator.is_translation_one_joint_bind_pose()==0 && navigator.is_translation_bind_pose()==0)
	{
	  navigator.enable_add_joint();
	  printf("[ADD Joint]\n");
	}
      else if(navigator.is_add_joint()==1)
	{
	  navigator.disable_add_joint();
	  printf("</ADD Joint>\n");
	}
      break;
    case GLUT_SPACE://space
      if(navigator.get_selection_mode()==1)
	{
	  float radius = 2;//0.05;
	  int closest_joint=0;


	  if(navigator.is_change_root()==1)
	    {
	      printf("ROOT CHANGED\n");
	      skeleton->new_father(navigator.get_sel_x(),navigator.get_sel_y(),navigator.get_sel_z());
	      *deformed_skeleton=*skeleton;
	      navigator.disable_change_root();

	      skinned.automatic_weight_calculation();
	      skinned.automatic_segmentation();
	      skinned.fill_color_segmentation();
	      *deformed_mesh=*mesh;
	      parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
	    }
	  else if(navigator.is_add_joint()==1 && navigator.get_selected_joint()!=-1)
	    {
	      skeleton->add_new_joint(navigator.get_sel_x(),navigator.get_sel_y(),navigator.get_sel_z(),navigator.get_selected_joint());
	      navigator.disable_add_joint();
	      cout<<"joint added\n"<<endl;
	      *deformed_skeleton=*skeleton;

	      skinned.automatic_weight_calculation();
	      skinned.automatic_segmentation();
	      skinned.fill_color_segmentation();
	      *deformed_mesh=*mesh;
	      parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
	    }
	  else
	    {

	      closest_joint = deformed_skeleton->find_closest_joint(navigator.get_sel_x(),navigator.get_sel_y(),navigator.get_sel_z(),radius);
	      printf("find joint %d\n",closest_joint);
	      if(closest_joint!=-1)
		navigator.set_selected_joint(closest_joint);
	      navigator.set_rotation_mode(0);
	    }

	  
	}
      else
	navigator.set_rotation_mode(0);
      break;
    case 'U':
      printf("[(%f,%f,%f) Closest vertex <%d>]\n",navigator.get_sel_x(),navigator.get_sel_y(),navigator.get_sel_z(),deformed_mesh->N_closest(navigator.get_sel_x(),navigator.get_sel_y(),navigator.get_sel_z(),1)[0]);
      break;

    case 'S':

      //parser.save_skeleton_sk("temp_skeleton.txt",*deformed_skeleton);
      parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
      printf("[Skeleton saved in temp_skeleton]\n");
      break;


    case 'T'://translation bind pose
      if(navigator.get_selected_joint()!=-1 && navigator.is_translation_one_joint_bind_pose()==0)
	{
	  cout<<"[Tranlation bind pose]\n"<<endl;
	  if(navigator.is_translation_bind_pose()==1)
	    {
	      navigator.disable_translation_bind_pose();
	      skinned.automatic_weight_calculation();
	      skinned.automatic_segmentation();
	      skinned.fill_color_segmentation();
	      *deformed_mesh=*mesh;
	      parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
	    }
	  else
	    navigator.enable_translation_bind_pose();
	}
      break;
    case 'Y'://translation bind pose only one joint
      if(navigator.get_selected_joint()!=-1 && navigator.is_translation_bind_pose()==0)
	{
	  if(navigator.is_translation_one_joint_bind_pose()==1)
	    {
	      printf("[Tranlation bind pose only this joint %d]\n",navigator.get_selected_joint());;
	      navigator.disable_translation_one_joint_bind_pose();
	      skinned.automatic_weight_calculation();
	      skinned.automatic_segmentation();
	      skinned.fill_color_segmentation();
	      *deformed_mesh=*mesh;
	      parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
	    }
	  else
	    navigator.enable_translation_one_joint_bind_pose();
	}
      break;
    case 'X'://delete joint
      if(navigator.get_selected_joint()!=-1)
	{
	  skeleton->destroy_joint(navigator.get_selected_joint());
	  *deformed_skeleton=*skeleton;
	  navigator.set_selected_joint(-1);
	  parser.save_collada_skeleton("data/temp_skeleton",*deformed_skeleton);
	}
      break;


    case 'W'://wireframe
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
      break;
    case 'F'://Filled
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      break;

    }

  glutPostRedisplay ();
}

static void motion_callback (int x, int y)
{
  /* si le bouton gauche est presse */
  double d_x=x-navigator.get_x_prec(),d_y=y-navigator.get_y_prec();
  double L=200;
  if (navigator.get_left_b()) 
    {
      if(navigator.get_current_view()==1)
	{
	  navigator.increase_theta_2(d_x);
	  navigator.increase_theta_1(d_y);
	}
      if(navigator.get_current_view()==2)
	{
	  if(navigator.get_selection_mode()==0)
	    {
	      navigator.set_tr2_x(navigator.get_tr2_x()-d_x*navigator.get_dist_obs2()/500);
	      navigator.set_tr2_y(navigator.get_tr2_y()+d_y*navigator.get_dist_obs2()/500);
	    }
	  else if(navigator.is_translation_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Matrix current = deformed_skeleton->get_joint(navigator.get_selected_joint())->get_matrix();
 	    current.set_translation(current.value(3,0)+(float)(d_x)/400.0f,current.value(3,1)-(float)(d_y)/400.0f,current.value(3,2));
 	    deformed_skeleton->get_joint(navigator.get_selected_joint())->set_matrix(current);
	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }
	  else if(navigator.is_translation_one_joint_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Joint *current = deformed_skeleton->get_joint(navigator.get_selected_joint());
	    Matrix world_current = current->get_world_matrix();
	    world_current.set_translation(world_current.value(3,0)+(float)(d_x)/400.0f,world_current.value(3,1)-(float)(d_y)/400.0f,world_current.value(3,2));
	    if(current->get_father()!=current)//special case for the father
	      current->set_matrix(world_current*(current->get_father()->get_world_matrix().get_inverse()));
	    else
	      current->set_matrix(world_current);
	    int k_son=0;
	    int N_son=current->get_N_son();
	    Joint *son=NULL;
	    for(k_son=0;k_son<N_son;k_son++)
	      {
		son = current->get_son(k_son);
		son->set_matrix(son->get_world_matrix()*(world_current.get_inverse()));
	      }

	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }
	  else if(navigator.get_selected_joint()>0 && navigator.get_rotation_mode()==1)
	    {
	      skeleton->set_angle(navigator.get_selected_joint(),0,0,skeleton->get_angle(navigator.get_selected_joint(),0,0)+d_x);
	    k_draw++;
	    if(k_draw%4==0)
	      deform_mesh();
	    }
	  else
	    {
	      navigator.increase_sel_x( -d_x*navigator.get_dist_obs2()/L);
	      navigator.increase_sel_y(  d_y*navigator.get_dist_obs2()/L);
	    }
	}
      if(navigator.get_current_view()==3)
	{
	  if(navigator.get_selection_mode()==0)
	    {
	      navigator.set_tr3_x(navigator.get_tr3_x()-d_x*navigator.get_dist_obs3()/500);
	      navigator.set_tr3_y(navigator.get_tr3_y()+d_y*navigator.get_dist_obs3()/500);
	    }

	  else if(navigator.is_translation_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Matrix current = deformed_skeleton->get_joint(navigator.get_selected_joint())->get_matrix();
 	    current.set_translation(current.value(3,0),current.value(3,1)-(float)(d_y)/400.0f,current.value(3,2)-(float)(d_x)/400.0f);
 	    deformed_skeleton->get_joint(navigator.get_selected_joint())->set_matrix(current);
	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }
	  else if(navigator.is_translation_one_joint_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Joint *current = deformed_skeleton->get_joint(navigator.get_selected_joint());
	    Matrix world_current = current->get_world_matrix();
	    world_current.set_translation(world_current.value(3,0),world_current.value(3,1)-(float)(d_y)/400.0f,world_current.value(3,2)-(float)(d_x)/400.0f);
	    if(current->get_father()!=current)//special case for the father
	      current->set_matrix(world_current*(current->get_father()->get_world_matrix().get_inverse()));
	    else
	      current->set_matrix(world_current);
	    int k_son=0;
	    int N_son=current->get_N_son();
	    Joint *son=NULL;
	    for(k_son=0;k_son<N_son;k_son++)
	      {
		son = current->get_son(k_son);
		son->set_matrix(son->get_world_matrix()*(world_current.get_inverse()));
	      }

	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }

	  else if(navigator.get_selected_joint()>0 && navigator.get_rotation_mode()==1)
	    {
	      skeleton->set_angle(navigator.get_selected_joint(),0,1,skeleton->get_angle(navigator.get_selected_joint(),0,1)+d_x);
	    k_draw++;
	    if(k_draw%4==0)
	      deform_mesh();
	    }
	  else
	    {
	      navigator.increase_sel_z(  d_x*navigator.get_dist_obs3()/L);
	      navigator.increase_sel_y(  d_y*navigator.get_dist_obs3()/L);
	    }
	}
      if(navigator.get_current_view()==4)
	{
	  if(navigator.get_selection_mode()==0)
	    {
	      navigator.set_tr4_x(navigator.get_tr4_x()-d_x*navigator.get_dist_obs4()/500);
	      navigator.set_tr4_y(navigator.get_tr4_y()+d_y*navigator.get_dist_obs4()/500);
	    }

	  else if(navigator.is_translation_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Matrix current = deformed_skeleton->get_joint(navigator.get_selected_joint())->get_matrix();
 	    current.set_translation(current.value(3,0)+d_x/400.0f,current.value(3,1),current.value(3,2)+d_y/400.0f);
 	    deformed_skeleton->get_joint(navigator.get_selected_joint())->set_matrix(current);
	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }
	  else if(navigator.is_translation_one_joint_bind_pose()==1 && navigator.get_selected_joint()!=-1)
	  {

	    Joint *current = deformed_skeleton->get_joint(navigator.get_selected_joint());
	    Matrix world_current = current->get_world_matrix();
	    world_current.set_translation(world_current.value(3,0)+d_x/400.0f,world_current.value(3,1),world_current.value(3,2)+d_y/400.0f);
	    if(current->get_father()!=current)//special case for the father
	      current->set_matrix(world_current*(current->get_father()->get_world_matrix().get_inverse()));
	    else
	      current->set_matrix(world_current);
	    int k_son=0;
	    int N_son=current->get_N_son();
	    Joint *son=NULL;
	    for(k_son=0;k_son<N_son;k_son++)
	      {
		son = current->get_son(k_son);
		son->set_matrix(son->get_world_matrix()*(world_current.get_inverse()));
	      }

	    deformed_skeleton->fix_init_bind_pose();
	    deformed_skeleton->precalculate_matrix();
	    *skeleton = *deformed_skeleton;
	  }

	  else if(navigator.get_selected_joint()>0 && navigator.get_rotation_mode()==1)
	    {
	      skeleton->set_angle(navigator.get_selected_joint(),0,2,skeleton->get_angle(navigator.get_selected_joint(),0,2)+d_x);
	    k_draw++;
	    if(k_draw%4==0)
	      deform_mesh();
	    }
	  else
	    {
	      navigator.increase_sel_x(  -d_x*navigator.get_dist_obs4()/L);
	      navigator.increase_sel_z(  -d_y*navigator.get_dist_obs4()/L);
	    }
	}


      navigator.set_x_prec(x);
      navigator.set_y_prec(y);
    }

  /* si le bouton droit est presse */
  if (navigator.get_right_b()) 
    {
      double u = (float)(y-navigator.get_y_prec());
      double fu = abs(u)/500.0;
      if(u<0)
	fu=-fu;
      
      if(navigator.get_current_view()==1)
	navigator.increase_dist_obs(fu);
      if(navigator.get_current_view()==2)
	navigator.increase_dist_obs2(fu);
      if(navigator.get_current_view()==3)
	navigator.increase_dist_obs3(fu);
      if(navigator.get_current_view()==4)
	navigator.increase_dist_obs4(fu);
	
      navigator.set_y_prec(y);
    }



  glutPostRedisplay ();
}
static void mouse_callback (int button, int action, int x, int y)
{

  if (button == GLUT_LEFT_BUTTON && action == GLUT_DOWN) {
    navigator.set_left_b(1);
    navigator.set_x_prec(x); 
    navigator.set_y_prec(y);
  }
  if (button == GLUT_LEFT_BUTTON && action == GLUT_UP) 
    navigator.set_left_b(0); 

  if (button == GLUT_RIGHT_BUTTON && action == GLUT_DOWN) {
    navigator.set_right_b(1);
    navigator.set_y_prec(y);
  }
  if (button == GLUT_RIGHT_BUTTON && action == GLUT_UP) 
    navigator.set_right_b(0); 


  if(x<navigator.get_X_screen()*navigator.get_percent_x() && y<navigator.get_Y_screen()*navigator.get_percent_y())
    navigator.set_current_view(1);
  if(x>navigator.get_X_screen()*navigator.get_percent_x() && y<navigator.get_Y_screen()*navigator.get_percent_y())
    navigator.set_current_view(3);
  if(x<navigator.get_X_screen()*navigator.get_percent_x() && y>navigator.get_Y_screen()*navigator.get_percent_y())
    navigator.set_current_view(2);
  if(x>navigator.get_X_screen()*navigator.get_percent_x() && y>navigator.get_Y_screen()*navigator.get_percent_y())
    navigator.set_current_view(4);





  glutPostRedisplay ();


}


int deform_mesh()
{
  *deformed_skeleton = *skeleton;
  skeleton->deform_skeleton(navigator.get_time(),deformed_skeleton);

  skinned.skinning();

  return 0;
}




int draw_scene()
{
  glEnable(GL_LIGHTING);

 
  glLineWidth(1.5);
  openGL_drawer.draw_mesh_slow(*deformed_mesh);

  glLineWidth(3.0);
  openGL_drawer.draw_skeleton(*skinned.get_deformed_skeleton());



  if(navigator.get_selection_mode()==1 && navigator.get_selected_joint()>=0)
    openGL_drawer.draw_selected_bone(*deformed_skeleton,navigator.get_selected_joint());




  return 0;
}

int draw_lines_screen()
{

  glViewport (0, 0, (int)(navigator.get_X_screen()), (int)(navigator.get_Y_screen()));

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  glOrtho(-1.0,1.0,-1.0,1.0,0.01,2.0);

  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glDisable(GL_LIGHTING);

  if(navigator.get_selection_mode()==1)
    glLineWidth(5.0);
  else
    glLineWidth(2.0);
  
  glBegin(GL_LINES);
  if(navigator.get_current_view()==2 || navigator.get_current_view()==4)
    {
      if(navigator.get_selection_mode()==1)
	glColor3f(1.0,0.0,0.0);
      else
	glColor3f(0.0,0.0,1.0);
    }
  else
    glColor3f(0.0,0.0,0.0);
  glVertex3f(navigator.get_percent_x()*2.0-1,-1.0,-1.0);
  glVertex3f(navigator.get_percent_x()*2.0-1, (1-navigator.get_percent_y())*2.0-1,-1.0);

  if(navigator.get_current_view()==1 || navigator.get_current_view()==3)
    {
      if(navigator.get_selection_mode()==1)
	glColor3f(1.0,0.0,0.0);
      else
	glColor3f(0.0,0.0,1.0);
    }
  else
    glColor3f(0.0,0.0,0.0);

  glVertex3f(navigator.get_percent_x()*2.0-1, (1-navigator.get_percent_y())*2.0-1,-1.0);
  glVertex3f(navigator.get_percent_x()*2.0-1, 1.0,-1.0);
  glEnd();




  glBegin(GL_LINES);
  if(navigator.get_current_view()==1 || navigator.get_current_view()==2)
    {
      if(navigator.get_selection_mode()==1)
	glColor3f(1.0,0.0,0.0);
      else
	glColor3f(0.0,0.0,1.0);
    }
  else
    glColor3f(0.0,0.0,0.0);

  glVertex3f(-1.0, (1-navigator.get_percent_y())*2.0-1,-1.0);
  glVertex3f( (navigator.get_percent_x())*2.0-1, (1-navigator.get_percent_y())*2.0-1,-1.0);


  if(navigator.get_current_view()==3 || navigator.get_current_view()==4)
    {
      if(navigator.get_selection_mode()==1)
	glColor3f(1.0,0.0,0.0);
      else
	glColor3f(0.0,0.0,1.0);
    }
  else
    glColor3f(0.0,0.0,0.0);

  glVertex3f( navigator.get_percent_x()*2.0-1, (1-navigator.get_percent_y())*2.0-1,-1.0);
  glVertex3f( 1, (1-navigator.get_percent_y())*2.0-1,-1.0);
  glEnd();



  return 0;
}


int draw_selection_mode()
{
  float L=99.9;
  glDisable(GL_DEPTH_TEST);
  glLineWidth(2.0);

  if(navigator.get_selection_mode()==1 || navigator.get_view_selection_mode())
    {
      glColor3f(0.9,0.9,0.9);



      glBegin(GL_LINES);

      glVertex3f(+L,navigator.get_sel_y(),navigator.get_sel_z());
      glVertex3f(-L,navigator.get_sel_y(),navigator.get_sel_z());

      glVertex3f(navigator.get_sel_x(),+L,navigator.get_sel_z());
      glVertex3f(navigator.get_sel_x(),-L,navigator.get_sel_z());

      glVertex3f(navigator.get_sel_x(),navigator.get_sel_y(),+L);
      glVertex3f(navigator.get_sel_x(),navigator.get_sel_y(),-L);

      glEnd();
    }


  glEnable(GL_DEPTH_TEST);
  return 0;

}




int load_model(const char* filename)
{

  File_parser parser;

  /// load file
  mesh->load_g_file(filename);
  parser.load_collada_skeleton("data/temp_skeleton.dae","0",skeleton);

  /// init color and normal
  mesh->build_normal_per_vertex();
  mesh->set_per_polygon_color();
  for(int k_polygon=0;k_polygon<mesh->polygon_number();k_polygon++)
    mesh->set_per_polygon_color(k_polygon,0.8,0.8,0.8,0.9);

  /// set skinning weights
  skinned.automatic_weight_calculation();
  skinned.automatic_segmentation();

  /// read and init skeleton
  skeleton->read_skeleton();
  skeleton->fix_init_bind_pose();
  skeleton->precalculate_matrix();

  /// fill the color
  skinned.fill_color_segmentation();



  return 0;

}


